<?php

class DimmeGooglemapsAdmin
{
    public function __construct()
    {
        add_action('init', array($this, 'init'));
        add_action('admin_menu', array($this, 'admin_menu'));
        add_action('add_meta_boxes', array($this, 'meta_box_googlemap'));
        add_action('save_post', array($this, 'save_post'));
    }

    public function meta_box_googlemap()
    {
        global $post;
        $post_type = get_post_type($post->ID);
        if (post_type_supports($post_type, 'dimme_googlemaps')) {
            add_meta_box('dimme-googlemaps-box', __('Googlemaps', 'dimme-googlemaps'), array($this, 'meta_box_googlemap_content'), $post_type, 'normal', 'core');
        }
    }

    public function meta_box_googlemap_content()
    {
        // Use nonce for verification
        wp_nonce_field(plugin_basename(__FILE__), 'dimme-googlemaps');
        global $post;

        $latitude = get_post_meta($post->ID, 'latitude', true);
        $longtitude = get_post_meta($post->ID, 'longtitude', true);

        // The actual fields for data entry
        echo '<label for="latitude">' . __("Latitude", 'dimme-googlemaps') . '</label> ';
        echo '<input type="text" id="latitude" name="latitude" value="' . $latitude . '" size="25" /> <br>';
        echo '<label for="longtitude">' . __("Longtitude", 'dimme-googlemaps') . '</label> ';
        echo '<input type="text" id="longtitude" name="longtitude" value="' . $longtitude . '" size="25" /> <br>';
        echo '<label for="searchAddress">' . __("Address", 'dimme-googlemaps') . '</label> ';
        echo '<input type="text" id="searchAddress" name="searchAddress" value="" size="50" /><button id="doSearchAddress">search</button> <br>';
        echo '<div id="dimme-googlemaps-map" style="width:100%;height:400px;"></div>';
    }

    public function save_post($post_id)
    {
        if (!isset($_POST['dimme-googlemaps']) || !wp_verify_nonce($_POST['dimme-googlemaps'], plugin_basename(__FILE__))) {
            return $post_id;
        }
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return $post_id;
        }

        // Check permissions
        if ('page' == $_POST['post_type']) {
            if (!current_user_can('edit_page', $post_id))
                return $post_id;
        } else {
            if (!current_user_can('edit_post', $post_id))
                return $post_id;
        }

        $latitude = $_POST["latitude"];
        $longtitude = $_POST["longtitude"];
        add_post_meta($post_id, 'latitude', $latitude, true) or update_post_meta($post_id, 'latitude', $latitude);
        add_post_meta($post_id, 'longtitude', $longtitude, true) or update_post_meta($post_id, 'longtitude', $longtitude);

        $googlemaps["item_date"] = date_i18n('Y-m-d H:i:s');
        $format[] = "%s";
        $googlemaps["post_type"] = $_POST["post_type"];
        $format[] = "%s";
        $googlemaps["latitude"] = $latitude;
        $format[] = "%f";
        $googlemaps["longtitude"] = $longtitude;
        $format[] = "%f";

        if ($_POST["post_ID"] == $post_id) {
            global $wpdb;
            $wpdb->query($wpdb->prepare("select * from `{$wpdb->prefix}dimme_googlemaps` where post_id = %d", $post_id));
            if ($wpdb->num_rows > 0) {
                $where = array("ID" => $wpdb->last_result[0]->ID);
                $whereFormat = array("%d");
                $wpdb->update($wpdb->prefix . "dimme_googlemaps", $googlemaps, $where, $format, $whereFormat);
            } else {
                $googlemaps["post_id"] = $post_id;
                $format[] = "%d";
                $wpdb->insert($wpdb->prefix . "dimme_googlemaps", $googlemaps, $format);
            }
        }
    }

    public function admin_menu()
    {
        add_options_page("Dimme Googlemap", "Dimme Googlemap", 'administrator', "dimme_googlemaps_admin_init", array($this, "admin_option_page"));
    }

    public function admin_option_page()
    {
        include('dimme-googlemaps-admin.php');
    }

    public function get_items()
    {
        $nonce = $_GET['postCommentNonce'];
        if (wp_verify_nonce($nonce, 'dimme-googlemap-post-comment-nonce')) {
            // mag
        } else {
            die ('Busted!');
        }

        $dimmeGooglemap = DimmeGooglemaps::get_instance();
        $basePostType = $dimmeGooglemap->get_post_type();

        $bounds = isset($_GET["bounds"]) ? $_GET["bounds"] : false;
        $type = isset($_GET["type"]) ? $_GET["type"] : $basePostType;
        $where = array();
        if ($bounds) {
            $bounds = explode(",", $bounds);
            if (count($bounds) == 4) {
                list($latitude1, $longtitude1, $latitude2, $longtitude2) = $bounds;
                $where[] = sprintf("(latitude >= %f AND latitude <= %f)", $latitude1, $latitude2);
                $where[] = sprintf("(longtitude >= %f AND longtitude <= %f)", $longtitude1, $longtitude2);
            }
        }
        if ($type) {
            $where[] = sprintf("post_type = '%s'", $type);
        }

        if (count($where) > 0) {
            global $wpdb;
            $wpdb->query($wpdb->prepare("select * from `wp_dimme_googlemaps` where " . implode(" AND ", $where), array()));

            $posts = array();
            foreach ($wpdb->last_result as $row) {
                $posts[$row->post_type][$row->post_id] = $row;
            }
            $postResult = array();
            foreach ($posts as $postType => $rows) {
                $postResult["items"][] = $postType;
                $postIn = array_keys($rows);
                $query = new WP_Query(array('post_type' => $postType, 'post_status' => 'publish', 'post__in' => $postIn));
                if ($query->have_posts()) {
                    $results = array();
                    while ($query->have_posts()) {
                        $query->the_post();
                        $postId = get_the_ID();

                        $result["post_id"] = $postId;
                        $result["title"] = get_the_title();
                        $result["permalink"] = get_permalink();
                        $result["html"] = sprintf('<h3>%s</h3><a href="%s">%s</a>', $result["title"], $result["permalink"], __('View', 'dimme-googlemaps'));
                        $result["latitude"] = $rows[$postId]->latitude;
                        $result["longtitude"] = $rows[$postId]->longtitude;
                        $result["icon"] = $dimmeGooglemap->get_icon(get_post_type($postId));
                        $result["bounds"] = $postType == $basePostType ? true : false;

                        $results[] = $result;
                    }
                    $postResult[$postType] = $results;
                }
            }
        } else {
            $postResult = array("error" => "no");
        }

        header('Content-type: application/json');
        echo json_encode($postResult);
        exit;
    }

    public function init()
    {
        add_action('wp_ajax_nopriv_dimme_googlemap_get_items', array($this, 'get_items'));
        add_action('wp_ajax_dimme_googlemap_get_items', array($this, 'get_items'));

        //wp_enqueue_script('jquery');
        wp_enqueue_script('dimme_simple_googlemap', WP_PLUGIN_URL . '/dimme-googlemaps/js/googlemap.js');
        wp_enqueue_script('dimme_googlemaps-jsapi', 'http://www.google.com/jsapi');
        wp_enqueue_script('dimme_googlemaps', 'http://maps.google.com/maps/api/js?sensor=false');
    }
}