<?php

/**
 * Class DimmeGooglemaps
 */
class DimmeGooglemaps
{
    /**
     * @var bool
     */
    private $map_is_loaded = false;


    /**
     * @var []
     */
    private $icon_types;
    /**
     * @var null
     */
    private static $instance = null;

    /**
     * @return DimmeGooglemaps|null
     */
    public static function get_instance()
    {
        if (null == self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * DimmeGooglemaps constructor.
     */
    public function __construct()
    {
        add_action('init', array($this, 'init'));
        add_action('pre_get_posts', array($this, 'pre_get_posts'));
        add_action('loop_start', array($this, 'loop_start'));

        add_filter('the_content', array($this, 'the_content'));
        add_filter('template_include', array($this, 'template_include'), 99);

        add_shortcode('dimme_googlemaps', array($this, 'shortcode_googlemap'));
        add_shortcode('dimme_googlemaps_item', array($this, 'shortcode_googlemap_item'));
    }

    /**
     * @return string
     */
    public function get_map($add_filter = true, $add_directions = false)
    {
        if (true === $this->map_is_loaded) {
            return '';
        }

        $this->map_is_loaded = true;

        $map = '';
        if ($add_filter) {
            $map .= $this->get_filter();
        }

        $map .= '<div id="dimme-googlemaps-map" data-autoload="true"></div>';

        if ($add_directions) {
            $map .= '<form id="dimme-googlemap-directions"><strong>Directions</strong><br>Address: <input type="text" id="searchAddress" name="address" value="" placeholder="street 10, city">';
            $map .= '<input type="submit" id="doSearchAddress" name="bb" value="view route"></form>';
            $map .= '<div id="direction-message"></div>';
            $map .= '<br style="clear:both;"><div id="directions"></div>';
        }

        return $map;
    }

    /**
     * @param bool $add_filter
     */
    public function show_map($add_filter = true)
    {
        echo $this->get_map($add_filter);
    }

    /**
     * @param $template
     * @return string
     */
    function template_include($template)
    {
        //return $template;

        if ($this->is_googlemap_posttype() || $this->is_googlemap_page()) {
            if (is_single() || is_search()) {
                // load default page
                return $template;
            } else {
                $template_name = 'archive-googlemap.php';
            }

            $new_template = locate_template(array($template_name), false);
            if ('' != $new_template) {
                return $new_template;
            }
        }

        return $template;
    }

    /**
     * @return string
     */
    private function get_filter()
    {
        $post_types = get_option('dimme_googlemaps_posttypes');
        $hide_checked = isset($_SESSION["dimme_googlemaps_type"]) ? $_SESSION["dimme_googlemaps_type"] : array();

        $filter = '';
        if (is_array($post_types)) {
            $filter .= '<form action="" method="get" class="dimme_googlemap filter form-inline"><div><strong>' . __('Filter: ', 'dimme_googlemaps') . '</strong>';
            foreach ($post_types as $post_type) {
                $checked = false;
                $filter .= sprintf('<div class="checkbox"><label for="dimme_googlemaps_type_%s">', $post_type);
                $filter .= sprintf('<input type="checkbox" value="%s" name="dimme_googlemaps_type[%s]" id="dimme_googlemaps_type_%s" %s>', $post_type, $post_type, $post_type, $checked);
                $filter .= sprintf('%s</label></div> ', $post_type);
            }
            $filter .= '</div></form>';
        }
        return $filter;
    }

    /**
     * @return array
     */
    public function load_icon_types()
    {
        $types = array(
            'default' => plugins_url('dimme-googlemaps') . '/icons/information.png',
            'googlemap' => plugins_url('dimme-googlemaps') . '/icons/information.png',
            'googlemap-single' => plugins_url('dimme-googlemaps') . '/icons/information-single.png',
        );

        $types = apply_filters('dimme_googlemap_icon_types', $types);
        return $types;
    }

    /**
     * @param string $post_type
     * @param bool $single
     * @return string
     */
    public function get_icon($post_type, $single = false)
    {
        if (null === $this->icon_types) {
            $this->icon_types = $this->load_icon_types();
        }
        if(true === $single) {
            if(isset($this->icon_types[$post_type . '-single'])) {
                return $this->icon_types[$post_type . '-single'];
            }
        }

        if(isset($this->icon_types[$post_type])) {
            return $this->icon_types[$post_type];
        }

        return $this->icon_types['default'];
    }

    /**
     * @return string
     */
    public function get_post_type()
    {
        $post_type = get_option('dimme_googlemaps_posttype');
        return $post_type ? $post_type : 'googlemap';
    }

    /**
     * @return bool
     */
    public function is_googlemap_posttype()
    {
        if (is_search()) {
            return false;
        }

        $post_type = get_option('dimme_googlemaps_posttype');
        if ($post_type) {
            return $post_type === get_post_type();
        } else {
            return 'googlemap' === get_post_type();
        }
    }

    /**
     * @return bool
     */
    public function is_googlemap_page()
    {
        if (is_page()) {
            $page_id = get_option('dimme_googlemaps_page_id');
            if ($page_id && is_page($page_id)) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param $query
     * @return bool
     */
    public function loop_start($query)
    {
        /* @var $query WP_Query */
        if ($query->is_main_query() && $this->is_googlemap_posttype()) {
            if ($query->is_single() || $query->is_search()) {
                return false;
            }

            $this->show_map();
        }
    }

    /**
     *
     */
    public function pre_get_posts()
    {
        /* @var $wp_query WP_Query */
        global $wp_query;
        if ($wp_query->is_main_query() && $wp_query->is_post_type_archive()) {
            $post_type = $wp_query->query_vars['post_type'];
            $post_types = get_option('dimme_googlemaps_posttypes');
            if (in_array($post_type, $post_types)) {
                $wp_query->set('posts_per_page', 500);
                $wp_query->set('orderby', 'title');
                $wp_query->set('order', 'ASC');
            }
        }
    }

    /**
     *
     */
    public function init()
    {
        if (!is_admin()) {
            wp_register_style('dimme_googlemap_style', WP_PLUGIN_URL . '/dimme-googlemaps/css/dimme-googlemap.css');
            wp_enqueue_style('dimme_googlemap_style');
            wp_enqueue_script('jquery');
            wp_enqueue_script('jquery_dimme_googlemap', WP_PLUGIN_URL . '/dimme-googlemaps/js/jquery.dimme-map.js', array('jquery'));
            wp_enqueue_script('dimme_simple_googlemap', WP_PLUGIN_URL . '/dimme-googlemaps/js/googlemap-site.js');
            wp_localize_script('dimme_simple_googlemap', 'DimmeAjax', array(
                    // URL to wp-admin/admin-ajax.php to process the request
                    'ajaxurl' => admin_url('admin-ajax.php'),
                    // generate a nonce with a unique ID "myajax-post-comment-nonce"
                    // so that you can check it later when an AJAX request is sent
                    'postCommentNonce' => wp_create_nonce('dimme-googlemap-post-comment-nonce'),
                )
            );

            wp_enqueue_script('dimme_googlemaps-jsapi', 'http://www.google.com/jsapi');
            wp_enqueue_script('dimme_googlemaps', 'http://maps.google.com/maps/api/js?sensor=false');
        }

        register_post_type("googlemap",
            array(
                'labels' => array(
                    'name' => __('googlemap', 'dimme_googlemaps'),
                    'singular_name' => __('Googlemap', 'dimme_googlemaps'),
                    'add_new' => __('Add googlemap', 'dimme_googlemaps'),
                    'add_new_item' => __('Add New googlemap', 'dimme_googlemaps'),
                    'edit' => __('Edit', 'dimme_googlemaps'),
                    'edit_item' => __('Edit googlemap', 'dimme_googlemaps'),
                    'new_item' => __('New googlemap', 'dimme_googlemaps'),
                    'view' => __('View googlemap', 'dimme_googlemaps'),
                    'view_item' => __('View googlemap', 'dimme_googlemaps'),
                    'search_items' => __('Search googlemap', 'dimme_googlemaps'),
                    'not_found' => __('No googlemaps found', 'dimme_googlemaps'),
                    'not_found_in_trash' => __('No googlemaps found in trash', 'dimme_googlemaps'),
                    'parent' => __('Parent googlemap', 'dimme_googlemaps')
                ),
                'description' => __('Default googlemap.', 'dimme_googlemaps'),
                'public' => true,
                'show_ui' => true,
                'capability_type' => 'post',
                'publicly_queryable' => true,
                'exclude_from_search' => false,
                'hierarchical' => false,
                'rewrite' => true,
                'query_var' => true,
                'supports' => array('title', 'revisions', 'editor', 'excerpt', 'thumbnail', 'comments', 'dimme_googlemaps'),
                'has_archive' => true,
                'show_in_nav_menus' => false,
                'taxonomies' => array('category'),
            )
        );

        $post_types = get_option('dimme_googlemaps_posttypes');
        if (is_array($post_types)) {
            foreach ($post_types as $post_type) {
                add_post_type_support($post_type, 'dimme_googlemaps');
            }
        }
    }

    /**
     * @param $atts
     * @param null $content
     * @return string
     */
    public function shortcode_googlemap_item($atts, $content = null)
    {
        $a = shortcode_atts(array(
            'post' => false
        ), $atts);

        if ($a['post'] == false) {
            return '';
        };

        return $this->get_map_item();
    }

    /**
     * Get a map for a single post. Add a direction view on in
     *
     * @param $atts
     * @param null $content
     * @return string
     */
    public function shortcode_googlemap($atts, $content = null)
    {
        $a = shortcode_atts(array(
            'post' => false
        ), $atts);

        if ($a['post'] == false) {
            return '';
        };

        $post = get_post($a['post']);
        if ($post instanceof WP_Post) {
            $map_item = $this->get_map_item();
            if ($map_item) {
                return $this->get_map(false, true) . $map_item;
            }
        }
    }

    /**
     * @param $content
     * @return string
     */
    public function the_content($content)
    {
        /* @var $wp_query WP_Query */
        global $wp_query;
        global $post;

        $post_type = get_post_type($post->ID);
        if (false === post_type_supports($post_type, 'dimme_googlemaps')) {
            return $content;
        }

        if ($wp_query->is_search()) {

        } elseif ($wp_query->is_single()) {
            $content = "[dimme_googlemaps post=" . get_the_ID() . "]" . $content;
        } elseif ($wp_query->is_page()) {
//            if ($this->is_googlemap_page()) {
//                $content = $content . $this->get_map(true);
//                $post_type = get_option("dimme_googlemaps_posttype");
//                $post_type = $post_type ? $post_type : "googlemap";
//                $args = array(
//                    'post_type' => $post_type,
//                    'posts_per_page' => 500,
//                    'orderby' => 'title',
//                    'order' => 'ASC'
//                );
//
//                $wp_query = new WP_Query($args);
//                if ($wp_query->have_posts()) :
//                    while ($wp_query->have_posts()) :
//                        /* @var $post WP_Post */
//                        $wp_query->the_post();
//
//                        $content .= '<div class="googlemaps">';
//                        $content .= '<h3 class="entry-title"><a href="' . get_permalink(get_the_ID()) . '">' . get_the_title() . "</a></h3>";
//                        $content .= $this->get_map_item();
//                        $content .= '</div>';
//                    endwhile;
//                endif;
//
//                wp_reset_query();
//            }
        } else {
            $content = $content . $this->get_map_item();
        }


        return $content;
    }

    /**
     * @return string
     */
    public function get_map_item()
    {
        $latitude = get_post_meta(get_the_ID(), 'latitude', true);
        $longtitude = get_post_meta(get_the_ID(), 'longtitude', true);
        $icon = $this->get_icon(get_post_type(), is_singular());
        // todo bound

        $bound = true;//get_post_meta(get_the_ID(), 'bound', true);
        $permalink = get_permalink(get_the_ID());
        $strMaps = '';

        if ($latitude && $longtitude) {
            $strMaps .= sprintf('<div class="maps" data-post-id="%d" data-latitude="%f" data-longtitude="%f" data-icon="%s" data-bound="%s">', get_the_ID(), $latitude, $longtitude, $icon, $bound ? "true" : "false");
            $strMaps .= sprintf('<strong>%s</strong><br>%s<br><br><a href="%s">naar pagina</a>', get_the_title(), addslashes(get_the_content()), $permalink);
            $strMaps .= sprintf('</div>');
            $strMaps .= sprintf('<div class="showmap"><a href="#content" class="googlemap-show-map">' . _e('view on the map', 'dimme_googlemaps') . '</a><br><br></div>');
        }

        return $strMaps;
    }
}

