/*!
 * jQuery Dimme Google Map
 * http://dev.dimme.nl
 * Copyright (c) 2013 Casper Houdé
 * Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
 */
( function ($) {
    'use strict';

    $.dimme = $.dimme || {version: '0.1'};
    $.dimme.googlemap = {
        conf: {
            // These are the defaults.
            api: false,
            mapitems: ".maps",
            showmap: ".showmap",
            autoload: false,
            urldata: false,
            formitems: false,
            latitude: false,
            longtitude: false,
            directions: "#searchAddress",
            zoom: 13,
            mapTypeId: 'roadmap'
        }
    };

    function Googlemap(root, conf) {
        // current instance
        var self = this;
        var fire = root.add(self);
        var map;
        var bounds = new google.maps.LatLngBounds();
        var marker = [];
        var infowindow = new google.maps.InfoWindow({});
        var directionDisplay;
        var directionsService;
        var directionMarker;

        if (root.data("autoload")) {
            conf.autoload = root.data("autoload");
        }
        if (root.data("directions")) {
            conf.directions = root.data("directions");
        }
        if (root.data("latitude")) {
            conf.latitude = root.data("latitude");
        }
        if (root.data("longtitude")) {
            conf.longtitude = root.data("longtitude");
        }

        function openMarker(m, andZoom) {
            directionMarker = m;
            infowindow.setContent(m.html);
            infowindow.open(map, m);

            if (andZoom) {
                map.panTo(m.position);
                if (map.zoom < 14) {
                    map.setZoom(14);
                }
            }
        }

        function calcRoute() {
            $("#direction-message").text("");
            if (typeof (directionMarker) == "undefined") {
                return false;
            }
            var start = $(conf.directions).val();
            var end = directionMarker.getPosition();
            var request = {
                origin: start,
                destination: end,
                travelMode: google.maps.DirectionsTravelMode.DRIVING
            };

            directionsService.route(request, function (response, status) {
                if (status == google.maps.DirectionsStatus.OK) {
                    directionDisplay.setDirections(response);
                } else {
                    $("#direction-message").text("address not found");
                }
            });
        }

        $.extend(self, {
            init: function () {
                var myOptions = {};
                myOptions.zoom = conf.zoom;
                myOptions.mapTypeId = conf.mapTypeId;

                map = new google.maps.Map($(root).get(0), myOptions);

                if (conf.mapitems && $(conf.mapitems).length > 0) {
                    self.initMapItems();
                } else {
                    self.initMapLatLng();
                }
            }
            , initMapLatLng: function () {
                if (google.loader.ClientLocation) {
                    var latt = google.loader.ClientLocation.latitude;
                    var longg = google.loader.ClientLocation.longitude;
                    var latlng = new google.maps.LatLng(latt, longg);
                } else if (conf.latitude && conf.longtitude) {
                    var latlng = new google.maps.LatLng(conf.latitude, conf.longtitude);
                } else {
                    var latlng = new google.maps.LatLng("52.0611", "4.4933");
                }
                map.setCenter(latlng);
            }
            , initMapItems: function () {
                if ($(conf.mapitems).length < 1) {
                    return false;
                }

                $(conf.mapitems).each(function () {
                    var lat = $(this).data("latitude");
                    var lng = $(this).data("longtitude");
                    var icon = $(this).data("icon");
                    var bound = $(this).data("bound");
                    var t = $(this).data("title");
                    var post_id = $(this).data("postId");
                    var html = $(this).html();

                    self.addMarker(lat, lng, t, html, icon, post_id, bound);
                    $(this).hide();
                });

                if ($(conf.mapitems).length > 1) {
                    self.fitBounds();
                } else {
                    if ($(conf.mapitems).length === 1) {
                        setTimeout(function () {
                            var post_id = $(conf.mapitems).eq(0).data('postId');
                            directionMarker = marker[post_id];
                            marker[post_id].setZIndex(10000);
                            map.setCenter(marker[post_id].getPosition());
                            map.setZoom(16);
                        }, 100);
                    }
                }
            }
            , addMarker: function (lat, lng, t, html, icon, post_id, bound, type) {
                if (typeof(marker[post_id]) != "undefined") {
                    return;
                }
                
                var latlng = new google.maps.LatLng(lat, lng);
                marker[post_id] = new google.maps.Marker({
                    icon: icon
                    , position: latlng
                    , map: map
                    , title: t
                    , html: html
                });
                if (typeof(type) != "undefined") {
                    marker[post_id].type = type;
                }
                google.maps.event.addListener(marker[post_id], 'click', function () {
                    openMarker(this);
                });

                if (bound == true) {
                    self.addBounds(latlng);
                }
            }
            , removeMarkers: function (o) {
                o = o || {};
                for (var i = 0; i < marker.length; i++) {
                    if (marker[i]) {
                        if (o.type) {
                            if (marker[i].type == o.type) {
                                marker[i].setMap(null);
                                marker[i] = false;
                            }
                        } else {
                            marker[i].setMap(null);
                        }
                    }
                }
                marker = [];
            }
            , addBounds: function (latlng) {
                bounds.extend(latlng);
            }
            , fitBounds: function () {
                map.fitBounds(bounds);
            }
            , loadJson: function (d) {
                if (d.length < 1) {
                    return false;
                }
                for (var i = 0; i < d.items.length; i++) {
                    var type = d.items[i];
                    var list = d[d.items[i]];
                    for (var j = 0; j < list.length; j++) {
                        var ll = list[j];
                        self.addMarker(ll.latitude, ll.longtitude, ll.title, ll.html, ll.icon, ll.post_id, ll.bound, type);
                    }
                }
            }
            , loadFormItems: function (type, check) {
                if (conf.urldata) {
                    if (check) {
                        var request = {};
                        if (conf.autoload) {
                            request.bounds = map.getBounds().toUrlValue();
                        }
                        request.type = type;
                        request.postCommentNonce = conf.urldata.postCommentNonce;
                        request.action = 'dimme_googlemap_get_items';

                        $.get(conf.urldata.ajaxurl, request, function (d) {
                            self.loadJson(d);
                        });
                    } else {
                        self.removeMarkers({"type": type});
                    }
                }
            }
            , initAutoLoad: function () {
                if (conf.urldata) {
                    google.maps.event.addListener(map, 'idle', function () {
                        var request = {};
                        request.bounds = map.getBounds().toUrlValue();
                        request.postCommentNonce = conf.urldata.postCommentNonce;
                        request.action = 'dimme_googlemap_get_items';

                        $.get(conf.urldata.ajaxurl, request, function (d) {
                            self.loadJson(d);
                        });
                    });
                }
            }
            , initDirections: function (w) {
                if ($(w).length > 0) {
                    directionDisplay = new google.maps.DirectionsRenderer();
                    directionsService = new google.maps.DirectionsService();
                    directionDisplay.setMap(map);
                    directionDisplay.setPanel(document.getElementById('directions'));

                    $($(w).get(0).form).submit(function () {
                        calcRoute();
                        return false;
                    });
                }
            }
            , initShowmap: function (w) {
                $(w).click(function () {
                    var maps = $(this).parent().find(".maps");
                    if (maps.length > 0) {
                        if (maps.data("postId")) {
                            var post_id = maps.data("postId");
                            openMarker(marker[post_id], true);
                        }
                    }
                });
            }
        });

        self.init();
        if (conf.formitems) {
            $(conf.formitems).click(function () {
                self.loadFormItems($(this).val(), this.checked);
            });
        }
        if (conf.autoload) {
            self.initAutoLoad();
        }
        if (conf.directions) {
            self.initDirections(conf.directions);
        }
        if (conf.showmap) {
            self.initShowmap(conf.showmap);
        }
    }

    $.fn.dimmegooglemap = function (opts) {
        var conf = $.extend({}, $.dimme.googlemap.conf, opts);

        this.each(function () {
            var el = new Googlemap($(this), conf);
            $(this).data("googlemap", el);
        });

        return conf.api ? el : this;

    };
}(jQuery) );